/* Icons
 ****************************************************************************/
/* Add some margin to icon images like profil, or logout. (these are not the
 * bootstrap icons) */
img.icon {
    margin: 0 5px;
}

/* Bootstrap overwrites
 ****************************************************************************/
/* Hide carret in dropup menus as they are displayed in up direction which
 * does not fit for the administrationial menu. So hide them all.
 * TODO: Display caret but in down direction. */
.navbar .nav > li.dropup > .dropdown-menu::after, 
.navbar .nav > li.dropup > .dropdown-menu::before {
  display:none;
}

/* Reduce bottom margin from 30 to 20 (was 20 0 30) to make the content
 * under the page-header more compact. */
.page-header {
  margin: 10px 0;
}
.page-header h1 {
  margin: 5px 0;
}
