"""ringo-0.5.0

Revision ID: 4d5e42cec5bc
Revises: None
Create Date: 2013-08-14 23:24:36.139171

"""

# revision identifiers, used by Alembic.
revision = '4d5e42cec5bc'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###
