/* Context menu
 * *************************************************************************/
/* Chane positon to align the menu with the page header on the left side. */
.context-menu {
  position: relative;
  top: 5px;
}

/* Status message
 * *************************************************************************/
/* Push the status message a littlebit down from the page header. */
#status-messages {
  margin-top: 10px;
}

/* Overview tables
 * *************************************************************************/
/* Change the mouse cursor to a pointer when hovering the rows of the
 * overviewtable to indicate that they are clickable. */
#data td:hover {
	cursor: pointer;
}

#data td a.filter {
  text-decoration: None;
  color: black;
}

.dataTables_filter input {
    margin-bottom: 0px;
}
.dataTables_filter {
    background-color: #F5F5F5;
    border: 1px solid #E3E3E3;
    border-radius: 3px 3px 3px 3px;
    padding: 9px;
    box-shadow: 0 1px 1px rgba(0, 0, 0, 0.05) inset;
    margin-bottom: 10px;
    min-height: 20px;
}

.table .table-toolbar {
    background-color: #F5F5F5;
}

/* Search
 * *************************************************************************/
/* Change the mouse cursor to a pointer when hovering the rows of the
 * overviewtable to indicate that they are clickable. */
.search-widget {
  margin-bottom: 10px;
}
.search-widget form {
  margin-bottom: 0px;
}
/* Overwrite the margin of the button (was 15px) */
.search-widget button {
    margin-right: 0px;
}
ul.dropdown-menu li table td a {
    padding: 10px 15px;
    color: black;
    text-decoration: None;
}
ul.dropdown-menu li table {
    margin-bottom: 0px;
}
ul.dropdown-menu li table td {
    vertical-align: middle;
}
